@@ -6,7 +6,9 @@
 #include <gio/gio.h>
 
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <stdlib.h>
 #include <time.h>
 
@@ -198,8 +200,9 @@ test_parsability (void)
 		{ "file:///tmp/file_doesnt_exist.wmv", FALSE, FALSE },
 		{ "http://live.hujjat.org:7860/main", FALSE, TRUE },
 		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, TRUE },
+		{ TEST_SRCDIR "HackerMedley", TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
 		{ NULL,  FALSE, FALSE }
-	};	
+	};
 
 	/* Loop through the list, downloading the URIs and checking for parsability */
 	for (i = 0; files[i].uri != NULL; ++i) {
@@ -271,6 +274,81 @@ parser_test_get_parse_result (const char *uri)
 	return ret;
 }
 
+static void
+entry_parsed_num_cb (TotemPlParser *parser,
+		     const char *uri,
+		     GHashTable *metadata,
+		     guint *ret)
+{
+	*ret = *ret + 1;
+}
+
+static guint
+parser_test_get_num_entries (const char *uri)
+{
+	TotemPlParserResult retval;
+	guint ret = 0;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", FALSE,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_num_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+playlist_started_order (TotemPlParser *parser,
+			const char *uri,
+			GHashTable *metadata,
+			gboolean *ret)
+{
+	*ret = TRUE;
+}
+
+static void
+entry_parsed_cb_order (TotemPlParser *parser,
+		       const char *uri,
+		       GHashTable *metadata,
+		       gboolean *ret)
+{
+	/* Check that the playlist started happened before the entry appeared */
+	g_assert (*ret != FALSE);
+}
+
+static gboolean
+parser_test_get_order_result (const char *uri)
+{
+	TotemPlParserResult retval;
+	gboolean pl_started;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "playlist-started",
+			  G_CALLBACK (playlist_started_order), &pl_started);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_cb_order), &pl_started);
+
+	pl_started = FALSE;
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return pl_started;
+}
+
 static TotemPlParserResult
 simple_parser_test (const char *uri)
 {
@@ -290,6 +368,166 @@ simple_parser_test (const char *uri)
 	return retval;
 }
 
+static void
+entry_parsed_genre_cb (TotemPlParser *parser,
+		 const char *uri,
+		 GHashTable *metadata,
+		 char **ret)
+{
+	if (*ret == NULL)
+		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE));
+}
+
+static char *
+parser_test_get_parse_genre (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_genre_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+playlist_started_title_cb (TotemPlParser *parser,
+			   const char *uri,
+			   GHashTable *metadata,
+			   char **ret)
+{
+	*ret = g_strdup (uri);
+}
+
+static char *
+parser_test_get_playlist_title (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "playlist-started",
+			  G_CALLBACK (playlist_started_title_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+test_itms_parsing (void)
+{
+	g_assert_cmpstr (parser_test_get_playlist_title ("itms://itunes.apple.com/gb/podcast/best-of-chris-moyles-enhanced/id142102961?ign-mpt=uo%3D4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moylesen/rss.xml");
+}
+
+static void
+entry_parsed_id_cb (TotemPlParser *parser,
+		 const char *uri,
+		 GHashTable *metadata,
+		 char **ret)
+{
+	if (*ret == NULL)
+		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_ID));
+}
+
+static char *
+parser_test_get_id (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_id_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+entry_parsed_download_cb (TotemPlParser *parser,
+		 const char *uri,
+		 GHashTable *metadata,
+		 char **ret)
+{
+	if (*ret == NULL)
+		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI));
+}
+
+static char *
+parser_test_get_download_uri (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_download_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+test_lastfm_parsing (void)
+{
+	char *uri;
+
+	g_test_bug ("625823");
+
+	uri = get_relative_uri (TEST_SRCDIR "old-lastfm-output.xspf");
+	g_assert_cmpstr (parser_test_get_download_uri (uri), ==, "http://freedownloads.last.fm/download/188024406/Kondratiev%2BWinter.mp3");
+	g_assert_cmpstr (parser_test_get_id (uri), ==, "d092a");
+	g_free (uri);
+
+	uri = get_relative_uri (TEST_SRCDIR "new-lastfm-output.xspf");
+	g_assert_cmpstr (parser_test_get_download_uri (uri), ==, "http://freedownloads.last.fm/download/402599273/Yellow.mp3");
+	g_assert_cmpstr (parser_test_get_id (uri), ==, "20a82");
+	g_free (uri);
+}
+
+static void
+test_parsing_xspf_genre (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "playlist.xspf");
+	g_assert_cmpstr (parser_test_get_parse_genre (uri), ==, "Test Genre");
+	g_free (uri);
+}
+
 static void
 test_parsing_rtsp_text_multi (void)
 {
@@ -312,7 +550,7 @@ test_parsing_rtsp_text (void)
 static void
 test_parsing_hadess (void)
 {
-	if (g_strcmp0 (g_get_user_name (), "hadess"))
+	if (g_strcmp0 (g_get_user_name (), "hadess") == 0)
 		g_assert (simple_parser_test ("file:///home/hadess/Movies") == TOTEM_PL_PARSER_RESULT_SUCCESS);
 }
 
@@ -337,6 +575,31 @@ test_parsing_broken_asx (void)
 	g_assert (result != TOTEM_PL_PARSER_RESULT_ERROR);
 }
 
+static void
+test_parsing_out_of_order_asx (void)
+{
+	char *uri;
+	gboolean result;
+
+	/* From http://82.135.234.195/pukas.wax */
+	uri = get_relative_uri (TEST_SRCDIR "pukas.wax");
+	result = parser_test_get_order_result (uri);
+	g_free (uri);
+	g_assert (result != FALSE);
+}
+
+static void
+test_parsing_num_entries (void)
+{
+	char *uri;
+	guint num;
+
+	uri = get_relative_uri (TEST_SRCDIR "missing-items.pls");
+	num = parser_test_get_num_entries (uri);
+	g_free (uri);
+	g_assert (num == 19);
+}
+
 static void
 test_parsing_404_error (void)
 {
@@ -386,6 +649,39 @@ test_parsing_xml_mixed_cdata (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_not_asx_playlist (void)
+{
+	char *uri;
+	g_test_bug ("610471");
+	/* File from https://bugzilla.gnome.org/show_bug.cgi?id=610471#c0 */
+	uri = get_relative_uri (TEST_SRCDIR "asf-with-asx-suffix.asx");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_parsing_not_really_php (void)
+{
+	char *uri;
+	g_test_bug ("590722");
+	/* File from http://startwars.org/dump/remote_xspf.php */
+	uri = get_relative_uri (TEST_SRCDIR "remote_xspf.php");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_parsing_not_really_php_but_html_instead (void)
+{
+	char *uri;
+	g_test_bug ("624341");
+	/* File from http://www.novabrasilfm.com.br/ao-vivo/audio.php */
+	uri = get_relative_uri (TEST_SRCDIR "audio.php");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_IGNORED);
+	g_free (uri);
+}
+
 #define MAX_DESCRIPTION_LEN 128
 #define DATE_BUFSIZE 512
 #define PRINT_DATE_FORMAT "%Y-%m-%dT%H:%M:%SZ"
@@ -544,12 +840,20 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
 		g_test_add_func ("/parser/parsing/404_error", test_parsing_404_error);
+		g_test_add_func ("/parser/parsing/out_of_order_asx", test_parsing_out_of_order_asx);
 		g_test_add_func ("/parser/parsing/xml_head_comments", test_parsing_xml_head_comments);
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
 		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
 		g_test_add_func ("/parser/parsing/single_line_rtsptext", test_parsing_rtsp_text);
 		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
 		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
+		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
+		g_test_add_func ("/parser/parsing/not_really_php", test_parsing_not_really_php);
+		g_test_add_func ("/parser/parsing/not_really_php_but_html_instead", test_parsing_not_really_php_but_html_instead);
+		g_test_add_func ("/parser/parsing/num_items_in_pls", test_parsing_num_entries);
+		g_test_add_func ("/parser/parsing/xspf_genre", test_parsing_xspf_genre);
+		g_test_add_func ("/parser/parsing/itms_link", test_itms_parsing);
+		g_test_add_func ("/parser/parsing/lastfm-attributes", test_lastfm_parsing);
 
 		return g_test_run ();
 	}
